<?php
/*
 * Service class for Workflow History
 *
 * @copyright   Copyright (c) 2016, Nugget Solutions, Inc
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       2.0
 *
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/*
 * OW_History_Service Class
 *
 * @since 2.0
 */
class OW_History_Service {

	/*
	 * Set things up.
	 *
	 * @since 2.0
	 */
	public function __construct() {
		add_action( 'wp_ajax_purge_workflow_history', array( $this, 'purge_history' ) );
	}

	/*
	 * Get Workflow History object from ID
	 *
	 * @param int $action_history_id
	 * @return OW_Action_History $action_history object
	 *
	 * @since 2.0
	 */
	public function get_action_history_by_id( $action_history_id ) {
		global $wpdb;

		// sanitize the data
		$action_history_id = intval( sanitize_text_field( $action_history_id ) );

		$result = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE ID = %d ORDER BY create_datetime DESC", $action_history_id ) );

		$action_history = $this->get_action_history_from_result_set( $result );
		return $action_history;

	}

	/*
	 * Get Workflow History object from "from_id" - the previous step_id
	 *
	 * @param int $from_id - previous step id
	 * @return OW_Action_History $action_history object
	 *
	 * @since 2.0
	 */
	public function get_action_history_by_from_id( $from_id ) {
		global $wpdb;

		// sanitize the data
		$from_id = intval( $from_id );

		$result = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE from_id = %d", $from_id ) );
		$action_history = $this->get_action_history_from_result_set( $result );
		return $action_history;
	}

	/*
	 * Get Workflow History object from action status for a post
	 *
	 * @param int $action_status
	 * @param int|null $post_id
	 * @return mixed OW_Action_History array $action_history object
	 *
	 * @since 2.0
	 */
	public function get_action_history_by_status( $action_status, $post_id ) {
		global $wpdb;
		if ( ! empty( $post_id ) ) {
			// sanitize the data
			$action_status = sanitize_text_field( $action_status ) ;
			$post_id = intval( $post_id );
			$action_histories = array();
			$results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE action_status = %s AND post_id = %d ORDER BY create_datetime DESC", $action_status, $post_id ) );
			foreach ( $results as $result ) {
				$action_history = $this->get_action_history_from_result_set( $result );
				array_push( $action_histories, $action_history );
			}
			return $action_histories;
		}
		return null;
	}

	/*
	 * Get Workflow History object from post
	 *
	 * @param int $post_id
	 * @return OW_Action_History $action_history object
	 *
	 * @since 2.0
	 */
	public function get_action_history_by_post( $post_id ) {
		global $wpdb;

		$post_id = intval( $post_id );
		$action_histories = array();
		$results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE post_id = %d ORDER BY create_datetime DESC", $post_id ) );
		foreach ( $results as $result ) {
			$action_history = $this->get_action_history_from_result_set( $result );
			array_push( $action_histories, $action_history );
		}
		return $action_histories;

	}

	/**
	 * Get Workflow History object from multiple parameters
	 *
    * @param string $action_status
    * @param int step_id
    * @param int $post_id
	 * @param int $from_id - previous step id
	 * @return OW_Action_History $action_history object
	 *
	 * @since 2.0
	 */
	public function get_action_history_by_parameters( $action_status, $step_id, $post_id, $from_id ) {
		global $wpdb;

		// sanitize the data
		$action_status = sanitize_text_field( $action_status ) ;
		$step_id = intval( $step_id ) ;
		$post_id = intval( $post_id );
		$from_id = intval( $from_id );

		$result = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE action_status = %s AND step_id = %d AND post_id = %d AND from_id = %d", $action_status, $step_id, $post_id, $from_id ) );

		$action_history = $this->get_action_history_from_result_set( $result );
		return $action_history;
	}

	/**
	 * Get Review History object by ID
	 *
	 * @param int $review_history_id
	 * @return OW_Review_History $review_history object
	 *
	 * @since 2.0
	 */
	public function get_review_action_by_id( $review_history_id ) {
		global $wpdb;

		$review_history_id = intval( $review_history_id );

		$result = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_table_name() . " WHERE ID = %d", $review_history_id ) );

		$review_history = $this->get_review_history_from_result_set( $result );
		return $review_history;
	}

	/**
	 * Get Review History object by action history id
	 *
	 * @param int $action_history_id
	 * @param string|null $order_by
	 * @return mixed OW_Review_History array
	 *
	 * @since 2.0
	 */
	public function get_review_action_by_history_id( $action_history_id, $order_by = null ) {
		global $wpdb;

		$action_history_id = intval( $action_history_id );
		if ( ! empty ( $order_by ) ) {
			$order_by = sanitize_text_field( $order_by );
		}

		$review_histories = array();

		if ( empty( $order_by ) ) {
			$order_by_clause = " ORDER BY ID DESC";
		} else if ( $order_by == 'update_datetime' ) {
			$order_by_clause = "ORDER BY (update_datetime ='0000-00-00 00:00:00') DESC, " . $order_by . " DESC ";
		} else {
			$order_by_clause = " ORDER BY " . $order_by . " DESC ";
		}
		$results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_table_name() . " WHERE action_history_id = %d " . $order_by_clause, $action_history_id ) );
		foreach ( $results as $result ) {
			$review_history = $this->get_review_history_from_result_set( $result );
			array_push( $review_histories, $review_history );
		}
		return $review_histories;
	}

	/**
	 * Get Review History object by review status
	 *
	 * @param string $review_status
	 * @param int $action_history_id
	 * @return mixed OW_Review_History array
	 *
	 * @since 2.0
	 */
	public function get_review_action_by_status( $review_status, $action_history_id ) {
		global $wpdb;

		$action_history_id = intval( $action_history_id );
		$review_status = sanitize_text_field( $review_status );

		$review_histories = array();

		$results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_table_name() . " WHERE review_status = %s AND action_history_id = %d ORDER BY ID DESC", $review_status, $action_history_id ) );
		foreach ( $results as $result ) {
			$review_history = $this->get_review_history_from_result_set( $result );
			array_push( $review_histories, $review_history );
		}
		return $review_histories;
	}

	/**
	 * Get Review History object by actor id
	 *
	 * @param int $review_history_id
	 * @return OW_Review_History $review_history object
	 *
	 * @since 2.0
	 */
	public function get_review_action_by_actor( $actor_id, $review_status, $action_history_id ) {
		global $wpdb;

		$actor_id = intval( $actor_id );
		$action_history_id = intval( $action_history_id );
		$review_status = sanitize_text_field( $review_status );

		$result = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM " . OW_Utility::instance()->get_action_table_name() . " WHERE review_status = %s AND actor_id = %d AND action_history_id = %d", $review_status, $actor_id, $action_history_id ) );
		$review_history = $this->get_review_history_from_result_set( $result );
		return $review_history;
	}


	/*
	 * function to result the action history object from the DB result set
	 *
	 * @since 2.0
	 */
	private function get_action_history_from_result_set( $result ) {
		if (! $result ) {
			return "";
		}
		$action_history = new OW_Action_History( );
		$action_history->ID = $result->ID;
		$action_history->action_status = $result->action_status;
		$action_history->comment = $result->comment;
		$action_history->step_id = $result->step_id;
		$action_history->assign_actor_id = $result->assign_actor_id;
		$action_history->post_id = $result->post_id;
		$action_history->from_id = $result->from_id;
		$action_history->due_date = $result->due_date;
		$action_history->reminder_date = $result->reminder_date;
		$action_history->reminder_date_after = $result->reminder_date_after;
		$action_history->create_datetime = $result->create_datetime;

		return $action_history;
	}

	/*
	 * function to result the review history object from the DB result set
	 *
	 * @since 2.0
	 */
	private function get_review_history_from_result_set( $result ) {
		if (! $result ) {
			return "";
		}
		$review_history = new OW_Review_History( );
		$review_history->ID = $result->ID;
		$review_history->review_status = $result->review_status;
		$review_history->comments = $result->comments;
		$review_history->step_id = $result->step_id;
		$review_history->next_assign_actors = $result->next_assign_actors;
		$review_history->actor_id = $result->actor_id;
		$review_history->due_date = $result->due_date;
		$review_history->action_history_id = $result->action_history_id;
		$review_history->update_datetime = $result->update_datetime;

		return $review_history;
	}

	public function get_table_header()
	{
		$order = ( isset($_GET['order']) && sanitize_text_field( $_GET["order"] ) == "desc" ) ? "asc" : "desc" ;

		if( isset($_GET['orderby']) && sanitize_text_field( $_GET["orderby"] ) == "post_title" )
			$post_order_class = sanitize_text_field( $_GET["order"] );
		else
			$post_order_class = "" ;

		if( isset($_GET['orderby']) && sanitize_text_field( $_GET["orderby"] ) == "wf_name" )
			$wf_order_class = sanitize_text_field( $_GET["order"] );
		else
			$wf_order_class = "" ;

		if( isset($_GET['orderby']) && sanitize_text_field( $_GET["orderby"] ) == "create_datetime" )
			$create_date_order_class = sanitize_text_field( $_GET["order"] );
		else
			$create_date_order_class = "" ;

		$where_post = ( isset($_GET['post']) && sanitize_text_field( $_GET["post"] )) ? "&post=" . intval( sanitize_text_field( $_GET["post"] )) : "" ;

		echo "<tr>";
		echo "<th scope='col' class='manage-column check-column' ><input type='checkbox'></th>";
		echo "<th scope='col' class='sorted $post_order_class'>
		<a href='admin.php?page=oasiswf-history&orderby=post_title&order=$order" . $where_post . "'>
					<span>". __("Title", "oasisworkflow") . "</span>
					<span class='sorting-indicator'></span>
				</a>
			</th>" ;
		echo "<th>" . __("Actor", "oasisworkflow") . "</th>" ;
		echo "<th scope='col' class='sorted $wf_order_class'>
		<a href='admin.php?page=oasiswf-history&orderby=wf_name&order=$order" . $where_post . "'>
					<span>". __("Workflow(version)", "oasisworkflow") . "</span>
					<span class='sorting-indicator'></span>
				</a>
			</th>" ;
		echo "<th>" . __("Step", "oasisworkflow") . "</th>";
		echo "<th scope='col' class='sorted $create_date_order_class'>
		<a href='admin.php?page=oasiswf-history&orderby=create_datetime&order=$order" . $where_post . "'>
					<span>". __("Assigned date", "oasisworkflow") . "</span>
					<span class='sorting-indicator'></span>
				</a>
			</th>" ;
		echo "<th scope='col'>". __("Sign off date", "oasisworkflow") . "</th>" ;
		echo "<th scope='col'>". __("Result", "oasisworkflow") . "</th>" ;
		echo "<th scope='col' class='history-comment'>". __("Comments", "oasisworkflow") . "</th>" ;
		echo "</tr>";
	}

	/**
	 * get all the workflow history
	 *
	 * @param null|int $post_id, if particular post id is provided, get history only for that post
	 * @return mixed $history result set
	 *
	 * @since 2.0
	 */
	static function get_workflow_history_all( $post_id = null )
	{
		global $wpdb;

		// use white list approach to set order by clause
		$order_by = array(
				'post_title' => 'post_title',
				'wf_name' => 'wf_name',
				'create_datetime' => 'create_datetime'
		);

		$sort_order = array(
				'asc' => 'ASC',
				'desc' => 'DESC',
		);

		if ( ! empty ( $post_id ) ) {
			$post_id = intval( $post_id );
		}

		// default order by
		$order_by_columns = " ORDER BY A.ID DESC"; // default order by column
		if ( isset($_GET['orderby']) && $_GET['orderby'] ) {
			// sanitize data
			$user_provided_order_by = sanitize_text_field( $_GET['orderby'] );
			$user_provided_order = sanitize_text_field( $_GET['order'] );
			if (  array_key_exists ( $user_provided_order_by, $order_by )) {
				$order_by_columns = " ORDER BY " . $order_by[$user_provided_order_by] . " " . $sort_order[$user_provided_order] . ", A.ID DESC";
			}
		}

		$where_clause = "action_status != 'complete' AND action_status != 'cancelled'" ;

		// if post id is provided, filter by post_id
		if( $post_id ) {
			$where_clause .= " AND post_id= %d ";
		}
		$sql = "SELECT A.* , B.post_title, C.ID as userid, C.display_name as assign_actor, D.step_info, D.workflow_id, D.wf_name, D.version
					FROM
					((SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE $where_clause) AS A
					LEFT JOIN
					{$wpdb->posts} AS B
					ON  A.post_id = B.ID
					LEFT JOIN
					{$wpdb->users} AS C
					ON A.assign_actor_id = C.ID
					LEFT JOIN
					(SELECT AA.*, BB.name as wf_name, BB.version FROM " . OW_Utility::instance()->get_workflow_steps_table_name() . " AS AA LEFT JOIN " . OW_Utility::instance()->get_workflows_table_name() . " AS BB ON AA.workflow_id = BB.ID) AS D
					ON A.step_id = D.ID)
					{$order_by_columns}" ;

					$result = "";
					if ( $post_id ) {
						$result = $wpdb->get_results( $wpdb->prepare( $sql, $post_id )) ;
					} else {
						$result = $wpdb->get_results( $sql ) ;
					}
		return $result;
	}

	/*
	 * get the count of history records
	 * @param int|null $post_id, if post id is provided, get count for the particular post only
	 * @return int count of history records
	 *
	 * @since 2.0
	 */
	public function get_workflow_history_count( $post_id = null )
	{
		global $wpdb;

		$where_clause = "action_status != 'complete' AND action_status != 'cancelled'" ;

		// if post id is provided, filter by post_id
		if( $post_id ) {
			$where_clause .= " AND post_id= %d ";
		}
		$sql = "SELECT A.*
					FROM
						((SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE $where_clause) AS A
							LEFT JOIN " . OW_Utility::instance()->get_action_table_name() . " AS C
						ON A.ID = C.action_history_id)
					" ;

		$results = "";
		if ( $post_id ) {
			$results = $wpdb->get_results( $wpdb->prepare( $sql, $post_id )) ;
		} else {
			$results = $wpdb->get_results( $sql ) ;
		}

		$final_results = array();
		foreach ($results as $result) {
			$final_results[] = $result;
		}
		return count( $final_results );
	}

	/*
	 * Purge or Delete history - ajax function
	 * Given a timeframe till all history till that time.
	 */
	public function purge_history( ) {
		global $wpdb;
		check_ajax_referer( 'owf-workflow-history', 'security' );

		if ( ! current_user_can( 'ow_delete_workflow_history' ) ) {
			wp_die( __( 'You are not allowed to delete workflow history.' ) );
		}

		$period = sanitize_text_field( $_POST["range"] );
		switch ( $period ){
			case 'one-month-ago' :
				$range = " AND posts.post_modified < DATE(curdate() - INTERVAL 1 MONTH) ";
				break;
			case 'three-month-ago' :
				$range = " AND posts.post_modified < DATE(curdate() - INTERVAL 3 MONTH) ";
				break;
			case 'six-month-ago' :
				$range = " AND posts.post_modified < DATE(curdate() - INTERVAL 6 MONTH) ";
				break;
			case 'twelve-month-ago' :
				$range = " AND posts.post_modified < DATE(curdate() - INTERVAL 12 MONTH) ";
				break;
			case 'everything' :
				$range = " ";
				break;
			default:
				return "not a valid period specified";
		}

		$sql = "SELECT posts.ID from " . $wpdb->posts . " AS posts WHERE 1=1 " . $range .
		" AND posts.id NOT IN " .
		"(SELECT A.post_id FROM
							(SELECT * FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE action_status = 'assignment') as A
							LEFT OUTER JOIN
							(SELECT * FROM " . OW_Utility::instance()->get_action_table_name() . " WHERE review_status = 'assignment') as B
							ON A.ID = B.action_history_id)";

		$posts_not_in_workflow_array = array();
		$posts_not_in_workflow_results = $wpdb->get_results( $sql ) ;
		if (empty( $posts_not_in_workflow_results )) {
			echo "success";
			die();
		}
		foreach ( $posts_not_in_workflow_results as $post_not_in_workflow ) {
			array_push($posts_not_in_workflow_array, $post_not_in_workflow->ID);
		}

		$int_place_holders = array_fill(0, count( $posts_not_in_workflow_array ), '%d');
		$place_holders_for_post_ids = implode(",", $int_place_holders);
		$sql = "SELECT ID FROM " . OW_Utility::instance()->get_action_history_table_name() . " WHERE 1=1 AND post_id in (" . $place_holders_for_post_ids . ")";
		$history_results = $wpdb->get_results( $wpdb->prepare($sql, $posts_not_in_workflow_array ) );

		// first delete any records from fc_action table
		if (empty( $history_results )) {
			echo "success";
			die();
		}
		$history_id_array = array();
		foreach ( $history_results as $history ) {
			array_push($history_id_array, $history->ID);
		}

		$int_place_holders = array_fill(0, count($history_id_array), '%d');
		$place_holders_for_history_ids = implode(",", $int_place_holders);

		// delete workflow history from action table
		$sql = "DELETE from " . OW_Utility::instance()->get_action_table_name() . " WHERE action_history_id in (" . $place_holders_for_history_ids . ")";
		$wpdb->get_results( $wpdb->prepare($sql, $history_id_array ) );

		// delete workflow history from action history table
		$sql = "DELETE from " . OW_Utility::instance()->get_action_history_table_name() . " WHERE id in (" . $place_holders_for_history_ids . ")";
		$wpdb->get_results( $wpdb->prepare( $sql, $history_id_array ) );

		echo "success";
		die();
	}

	public function history_graphic_box(){
		include( OASISWF_PATH . "includes/pages/subpages/history-graphic.php" ) ;
	}
}

// construct an instance so that the actions get loaded
$ow_history_service = new OW_History_Service();
?>