<?php

/*
 * Utilities class for Oasis Workflow
 *
 * @copyright   Copyright (c) 2016, Nugget Solutions, Inc
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       2.0
 *
 */

// Exit if accessed directly
if( !defined( 'ABSPATH' ) )
   exit;

/*
 * Utilities class - singleton class
 *
 * @since 2.0
 */

class OW_Utility {

   /**
    * Private ctor so nobody else can instance it
    *
    */
   private function __construct() {

   }

   /**
    * Call this method to get singleton
    *
    * @return singleton instance of OW_Utility
    */
   public static function instance() {

      static $instance = NULL;
      if( is_null( $instance ) ) {
         $instance = new OW_Utility();
      }

      return $instance;
   }

   /*
    * logging utility function
    * prints log statements in debug.log is logging is turned on in wp-config.php
    *
    * @since 2.0
    */

   public function logger( $message ) {
      if( WP_DEBUG === true ) {
         if( is_array( $message ) || is_object( $message ) ) {
            error_log( print_r( $message, true ) );
         } else {
            error_log( $message );
         }
      }
   }

   /*
    * get workflow table name
    *
    * @return string
    *
    * @since 2.0
    */

   public function get_workflows_table_name() {
      global $wpdb;
      return $wpdb->prefix . "fc_workflows";
   }

   /*
    * get workflow steps table name
    *
    * @return string
    *
    * @since 2.0
    */

   public function get_workflow_steps_table_name() {
      global $wpdb;
      return $wpdb->prefix . "fc_workflow_steps";
   }

   /*
    * get workflow history table name
    *
    * @return string
    *
    * @since 2.0
    */

   public function get_action_history_table_name() {
      global $wpdb;
      return $wpdb->prefix . "fc_action_history";
   }

   /*
    * get workflow history action table name
    *
    * @return string
    *
    * @since 2.0
    */

   public function get_action_table_name() {
      global $wpdb;
      return $wpdb->prefix . "fc_action";
   }

   /*
    * get workflow emails table name
    *
    * @return string
    *
    * @since 2.0
    */

   public function get_emails_table_name() {
      global $wpdb;
      return $wpdb->prefix . "fc_emails";
   }

   /*
    * returns integer value of the date
    *
    * @param string $date input date
    * @param string $date_separator example, "-" in 10-2-2014"
    * @return int integer value of the date
    */

   public function get_date_int( $date = null, $date_separator = "-" ) {
      $date = ( $date ) ? $date : current_time( 'mysql', 0 );
      $arr = explode( $date_separator, $date );
      return $arr[0] * 10000 + $arr[1] * 100 + $arr[2] * 1;
   }

   /*
    * formats the date to OASISWF_EDIT_DATE_FORMAT
    *
    * @param string $date string to be formatted
    * @return string formatted date
    *
    * @since 2.0
    */

   public function format_date_for_display_and_edit( $date ) {
      if( $date == "0000-00-00" ) {
         return "";
      }
      if( $date ) {
         $formatted_date = mysql2date( OASISWF_EDIT_DATE_FORMAT, $date, false );
         return $formatted_date;
      }
   }

   /*
    * displays the date in current/site format
    *
    * @param string $date string to be formatted
    * @param string $date_form could be date or datetime
    * @return string formatted date
    */

   public function format_date_for_display( $date, $format = "-", $date_form = "date" ) {
      // only date
      if( $date_form == "date" ) {
         if( $date == "0000-00-00" ) {
            return "";
         }
         if( $date ) {
            $formatted_date = mysql2date( get_option( 'date_format' ), $date );
            return $formatted_date;
         }
      } else { //date and time both
         $date_time = explode( " ", $date );

         if( $date_time[0] == "0000-00-00" ) {
            return "";
         }

         if( $date_time[0] ) {
            $date_time_format = get_option( 'date_format' ) . " " . get_option( 'time_format' );
            $formatted_date = mysql2date( $date_time_format, $date );
            return $formatted_date;
         }
      }
   }

   /*
    * create drop down for roles
    *
    * @param int $selected if passed, that will be selected by default
    * @param boolean $add_post_author_role, adds the custom Post Author role to the drop drop list
    * @return HTML for roles
    *
    * @since 2.0
    */

   public function owf_dropdown_roles_multi( $selected = null, $add_post_author_role = false ) {
      global $wpdb;
      $row = '';
      $selected_row = '';
      $roles_array = array();
      $post_author_role_name = __( "Post Author", "oasisworkflow" );
      $post_author_role_value = "owfpostauthor";

      // for single site
      $editable_roles = get_editable_roles();

      foreach ( $editable_roles as $role => $details ) {
         $name = translate_user_role( $details['name'] );
         if( is_array( $selected ) && in_array( esc_attr( $role ), $selected ) ) // preselect specified role
            $selected_row .= "\n\t<option selected='selected' value='" . esc_attr( $role ) . "'>$name</option>";
         else
            $row .= "\n\t<option value='" . esc_attr( $role ) . "'>$name</option>";
      }

      // add the custom post author role to the list
      if( $add_post_author_role ) {

         if( is_array( $selected ) && in_array( esc_attr( $post_author_role_value ), $selected ) ) // preselect specified role
            $selected_row .= "\n\t<option selected='selected' value='" . esc_attr( $post_author_role_value ) . "'>$post_author_role_name</option>";
         else
            $row .= "\n\t<option value='" . esc_attr( $post_author_role_value ) . "'>$post_author_role_name</option>";
      }
      echo $selected_row . $row;
   }

   /*
    * create HTML checkbox section for post types
    *
    * @param string $list_name name of the control/element
    * @param string $selected
    *
    * @return HTML for post type checkbox
    *
    * @since 2.0
    */

   public function owf_checkbox_post_types_multi( $list_name, $selected ) {
      $selected_row = '';
      // get all custom types
      $types = OW_Utility::instance()->owf_get_post_types();
      $checked = '';

      foreach ( $types as $post_type ) {
         // If post type is wordpress builtin then ignore it.
         if( is_array( $selected ) && in_array( esc_attr( $post_type['name'] ), $selected ) ) { // preselect specified role
            $checked = " ' checked='checked' ";
         } else {
            $checked = '';
         }


         $selected_row .= "<label style='display: block;'> <input type='checkbox' class='owf-checkbox'
					name='" . $list_name . "' value='" . esc_attr( $post_type['name'] ) . "'" . $checked . "/>";
         $selected_row .= $post_type['label'];
         $selected_row .= "</label>";
      }
      echo $selected_row;
   }

   /*
    * create HTML drop down section for post types
    *
    * @param string $list_name name of the control/element
    * @param string $selected
    *
    * @return HTML for post type drop down
    *
    * @since 2.0
    */

   public function owf_dropdown_post_types_multi( $selected_option ) {
      $selected_row = '';
      // get all custom types
      $types = OW_Utility::instance()->owf_get_post_types();

      $selected = '';
      foreach ( $types as $post_type ) {
         // If post type is wordpress builtin then ignore it.
         if( !empty( $selected_option ) && $post_type['name'] == $selected_option ) { // preselect specified role
            $selected = " selected='selected' ";
         } else {
            $selected = '';
         }

         $selected_row .= "<option value=" . esc_attr( $post_type['name'] ) . $selected . ">";
         $selected_row .= $post_type['label'];
         $selected_row .= "</option>";
      }
      echo $selected_row;
   }

   /*
    * TODO: Stop Gap arrangement to get all the post types from all the slave sites, by marking them public = true and show_ui = false in the main site
    * get_post_types is not multi-site aware
    * need to add a hook to let the users implement it and get a list of post types
    * We do not support attachments yet
    */

   public function owf_get_post_types() {
      global $wpdb;
      $all_types = array();
      $types = get_post_types( array( 'show_ui' => true ), 'objects' );
      foreach ( $types as $post_type ) {
         if( $post_type->name != 'attachment' ) {
            $temp_post_type = array( "name" => $post_type->name, "label" => $post_type->label );
            if( in_array( $temp_post_type, $all_types ) ) {
               continue;
            } else {
               array_push( $all_types, $temp_post_type );
            }
         }
      }

      // public = true, but, show_ui = false
      $types = get_post_types( array( 'show_ui' => false, 'public' => true ), 'objects' );
      foreach ( $types as $post_type ) {
         if( $post_type->name != 'attachment' ) {
            $temp_post_type = array( "name" => $post_type->name, "label" => $post_type->label );
            if( in_array( $temp_post_type, $all_types ) ) {
               continue;
            } else {
               array_push( $all_types, $temp_post_type );
            }
         }
      }

      return $all_types;
   }

   /**
    * get dropdown of post status
    * @param array $selected
    * @return HTML for post status drop down
    * @since 2.0
    */
   public static function owf_dropdown_post_status_multi( $selected ) {
      $row = '';
      $selected_row = '';
      $status_array = get_post_stati( array( 'show_in_admin_status_list' => true ), 'objects' );
      foreach ( $status_array as $status_id => $status_object ) {
         if( $status_id == "future" ) { // we do not consider future (scheduled posts) to be valid of auto submit
            continue;
         }
         if( is_array( $selected ) && in_array( $status_id, $selected ) ) // preselect specified status
            $selected_row .= "\n\t<option selected='selected' value='" . $status_id . "'>$status_object->label</option>";
         else
            $row .= "\n\t<option value='" . $status_id . "'>$status_object->label</option>";
      }
      echo $selected_row . $row;
   }

   /*
    * returns the pagination
    *
    * @param int $total_posts
    * @param int $current_page_number
    * @param int $per_page
    * @param string $action
    * @return HTML for displaying pagination
    *
    * @since 2.0
    */

   public function get_page_link( $total_posts, $current_page_number, $per_page = 20, $action = null ) {
      $allpages = ceil( $total_posts / $per_page );
      $base = "";
      if( empty( $action ) ) {
         $base = esc_url_raw( @add_query_arg( 'paged', '%#%' ) );
      } else if( !empty( $action ) ) {
         $base .= esc_url_raw( @add_query_arg( array( 'paged' => '%#%', 'action' => $action ) ) );
      }
      $page_links = paginate_links( array(
          'base' => $base,
          'format' => '',
          'total' => $allpages,
          'current' => $current_page_number
              ) );

      $page_links_text = sprintf( '<span class="displaying-num">' . __( 'Displaying %s-%s of %s', "oasisworkflow" ) . '</span>%s', number_format_i18n( ( $current_page_number - 1 ) * $per_page + 1 ), number_format_i18n( min( $current_page_number * $per_page, $total_posts ) ), number_format_i18n( $total_posts ), $page_links );

      echo $page_links_text;
   }

   /*
     public function get_post( $post_id )
     {
     global $wpdb;
     $post = null;
     if (function_exists('is_multisite') && is_multisite()) // to account for multisite
     {
     $blog_ids = $wpdb->get_col("SELECT blog_id FROM {$wpdb->base_prefix}blogs");
     foreach ( $blog_ids as $blog_id )
     {
     switch_to_blog( $blog_id );
     $post = get_post($post_id);
     if (!empty( $post)) {
     restore_current_blog();
     return $post;
     }
     restore_current_blog();
     }
     }

     $post = get_post($post_id);
     return $post;
     }
    */

   /**
    * Get the currently registered user
    */
   public function get_current_user() {
      global $wpdb;
      if( function_exists( 'wp_get_current_user' ) ) {
         return wp_get_current_user();
      } else if( function_exists( 'get_currentuserinfo' ) ) {
         global $userdata;
         get_currentuserinfo();
         return $userdata;
      } else {
         $user_login = sanitize_text_field( $_COOKIE[USER_COOKIE] );
         $current_user = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM {$wpdb->base_prefix}users WHERE user_login = '%s'", $user_login ) );
         return $current_user;
      }
   }

   /*
    * Utility function to get the current user's role
    *
    * @since 2.0
    */

   public function get_current_user_role() {
      global $wp_roles;
      foreach ( $wp_roles->role_names as $role => $name ) :
         if( current_user_can( $role ) )
            return $role;
      endforeach;
   }

   /*
    * Utility function to get the role of the a given user_id
    * @param int $user_id
    *
    * @since 2.0
    */

   public function get_user_role( $user_id ) {
      global $wp_roles;

      // sanitize the data
      $user_id = intval( $user_id );

      foreach ( $wp_roles->role_names as $role => $name ) :
         if( user_can( $user_id, $role ) )
            return $role;
      endforeach;
   }

   /*
    * given the user_id, get the full name of the user (could return nickname too)
    *
    * @param int $user_id
    * @return string display name of the user
    *
    * @since 2.0
    */

   public function get_user_name( $user_id ) {
      // sanitize the data
      $user_id = intval( $user_id );

      $user = get_userdata( $user_id );
      if( $user )
         return $user->data->display_name;
   }

   public function get_users_by_role( $role, $post_id = null ) {
      global $wpdb;
      if( count( $role ) > 0 ) {
         $user_string = array();
         $post_author_id = "";
         // Instead of using WP_User_Query, we have to go this route, because user role editor
         // plugin has implemented the pre_user_query hook and excluded the administrator users to appear in the list

         if( $post_id != null ) {
            $post = get_post( $post_id );
            $post_author_id = $post->post_author;
         }


         foreach ( $role as $k => $v ) {
            if( $k == 'owfpostauthor' ) { // this is a custom role added by oasis workflow
               $author_user = new WP_User( $post_author_id );
               $users = array( $author_user );
            } else {
               $user_role = '%' . $k . '%';
               $users = $wpdb->get_results( $wpdb->prepare( "SELECT users_1.ID, users_1.display_name FROM {$wpdb->base_prefix}users users_1
   				INNER JOIN {$wpdb->base_prefix}usermeta usermeta_1 ON ( users_1.ID = usermeta_1.user_id )
   				WHERE (usermeta_1.meta_key = '{$wpdb->prefix}capabilities' AND CAST( usermeta_1.meta_value AS CHAR ) LIKE %s)", $user_role ) );
            }
            foreach ( $users as $user ) {
               $current_user = get_current_user_id();
               $user_obj = new WP_User( $user->ID );
               if( !empty( $user_obj->roles ) && is_array( $user_obj->roles ) ) {
                  foreach ( $user_obj->roles as $user_role ) {
                     if( $user_role == $k || 'owfpostauthor' == $k ) { // if the selected role is 'postauthor'- the custom role.
                        $part["ID"] = $user->ID;

                        if( $user->ID == $post_author_id ) {
                           $part["name"] = $user->display_name . ' (Post Author)';
                        } else {
                           $part["name"] = $user->display_name;
                        }
                        // check if the user already exists in the available list
                        $exists = false;
                        foreach ( $user_string as $available_user ) {
                           if( $part["ID"] == $available_user->ID ) {
                              $exists = true; // the user exists, so lets not add it again.
                           }
                        }
                        if( !$exists ) {
                           $user_string[] = (object) $part;
                        }

                        break;
                     }
                  }
               }
            }
         }
         return (object) $user_string;
      }
      return "";
   }

   /*
    * Get next or previous date depending on the number of days
    * @param $input_date
    * @param $direction "next" or "pre"
    * @param $days number of days
    *
    * @return date in Y-m-d format
    *
    * @since 2.0
    */

   public function get_pre_next_date( $input_date, $direction = "next", $days = 1 ) {
      $date = new DateTime( $input_date );

      $date_stamp = $date->format( "U" );

      if( $direction == "next" )
         $date_stamp = $date_stamp + 3600 * 24 * $days;
      else if( $direction == "pre" )
         $date_stamp = $date_stamp - 3600 * 24 * $days;

      return gmdate( "Y-m-d", $date_stamp );
   }

   /**
    * Convert a date format to a jQuery UI DatePicker format
    *
    * @param string $dateFormat a date format
    * @return string
    */
   public function owf_date_format_to_jquery_ui_format( $dateFormat ) {

      $chars = array(
          // Day
          'd' => 'dd', 'j' => 'd', 'l' => 'DD', 'D' => 'D',
          // Month
          'm' => 'mm', 'n' => 'm', 'F' => 'MM', 'M' => 'M',
          // Year
          'Y' => 'yy', 'y' => 'y'
      );

      return strtr( (string) $dateFormat, $chars );
   }

   public function str_array_pos( $string, $array ) {
      for ( $i = 0, $n = count( $array ); $i < $n; $i ++ ) {
         if( stristr( $string, $array[$i] ) !== false ) {
            return true;
         }
      }
      return false;
   }

   public function is_post_tag_in_array( $post_id, $keyword_array ) {
      $post_tags = wp_get_post_tags( $post_id );
      $search_array = array_map( 'strtolower', $keyword_array );
      foreach ( $post_tags as $post_tag ) {
         if( in_array( strtolower( $post_tag->name ), $search_array ) ) {
            return true;
         }
      }

      return false;
   }

   public function is_post_category_in_array( $post_id, $keyword_array ) {
      $post_categories = wp_get_post_categories( $post_id );
      $search_array = array_map( 'strtolower', $keyword_array );
      foreach ( $post_categories as $post_category ) {
         $cat = get_category( $post_category );
         if( in_array( strtolower( $cat->name ), $search_array ) ) {
            return true;
         }
      }

      return false;
   }

   /*
    * DB related functions
    */

   /*
    * returns date formatted for DB
    *
    * @param string OASISWF_EDIT_DATE_FORMAT date format
    * @return string date formatted for db
    *
    * @since 2.0
    */

   public function format_date_for_db_wp_default( $formatted_date ) {
      $date = DateTime::createFromFormat( OASISWF_EDIT_DATE_FORMAT, $formatted_date );
      $date_with_mysql_format = $date->format( 'Y-m-d' );
      return $date_with_mysql_format;
   }

   /*
    * Given the table name and data will insert into the table and return the inserted id
    *
    * @param string $table - name of the table
    * @param mixed $data - data array to be inserted // should match the table format
    */

   public function insert_to_table( $table, $data ) {
      global $wpdb;
      $result = $wpdb->insert( $table, $data );
      if( $result ) {

         $row = $wpdb->get_row( "SELECT max(ID) as maxid FROM $table" );
         if( $row )
            return $row->maxid;
         else
            return false;
      } else {
         return false;
      }
   }

   /**
    * Show message on admin section relavant to plugin
    *
    * @param array $data
    * @return string
    *
    * @since 2.0
    */
   public function admin_notice( $data = array() ) {
      extract( $data ); // Extracts $message and $type from $data array
      switch ( $type ) {
         case 'error':
            $return = "<div id=\"message\" class=\"error\">\n";
            break;
         case 'update':
            $return = "<div id=\"message\" class=\"updated\">\n";
            break;
         case 'update-nag':
            $return = "<div id=\"message\" class=\"update-nag\">\n";
            break;
         default:
            $message = __( 'There\'s something wrong with your code...', 'owacfcompare' );
            $return = "<div id=\"message\" class=\"error\">\n";
            break;
      }

      $return .= "    <p>" . __( $message, 'owacfcompare' ) . "</p>\n";
      $return .= "</div>\n";
      return $return;
   }

   /**
    * Get username and its role
    * @access public
    * @param int $user_id
    * @return object
    * @since 2.0
    */
   public function get_user_role_and_name( $user_id ) {
      $user = new WP_User( $user_id );
      return (object) array( 'role' => trim( ucwords( array_shift( $user->roles ) ) ), 'username' => $user->display_name );
   }

   public function owf_pro_features() {
      $str = "";
      $show_upgrade_notice = get_site_option( 'oasiswf_show_upgrade_notice' );
      if( $show_upgrade_notice == "yes" ) {
         $str = '<div style="width:90%; float:left;  margin: 0px 50px 5px 7px; padding: 10px 10px 10px 10px; border: 1px solid #ddd; background-color:#FFFFE0;">
					   <form id="hide_notice">
						<div class="oasis_button_div">
								<a class="oasis_button" target="_blank" href="https://www.oasisworkflow.com/pricing-purchase"> ' . __( 'Learn More', 'oasisworkflow' ) . '</a>' .
                 '</div>
	      			<div style="width:80%; float:left; align: center">' .
                 '<p>' .
                 'Looking for additional features like "Ability to create multiple workflows", ' .
                 '<a target="_blank" href="https://www.oasisworkflow.com/workflow-support-for-updating-published-content">"Revise published content and add workflow support to revised content"</a>, ' .
                 '<a target="_blank" href="http://www.oasisworkflow.com/auto-submit-to-workflow">"Auto Submit"</a>, ' .
                 '<a target="_blank" href="https://www.oasisworkflow.com/extensions/oasis-workflow-teams">"Teams"</a>, ' .
                 '<a target="_blank" href="https://www.oasisworkflow.com/extensions/oasis-workflow-editorial-comments">"Contextual Comments"</a>, ' .
                 '<a target="_blank" href="https://www.oasisworkflow.com/extensions/oasis-workflow-front-end-actions">"Front End shortcodes"</a>, ' .
                 ' and much more...' .
                 '<p/><p>' .
                 '<strong><a target="_blank" href="https://www.oasisworkflow.com/request-a-demo-site">Request a demo site to try out the "Pro" features</a></strong>' .
                 '</p><p style="float:right;">
						 	<a href="admin.php?page=oasiswf-admin&action=hideNotice">' . 'I know, don\'t bug me.' .
                 '</a></p>
	              	</div></form>
	             </div>';
      }
      echo $str;
   }

}

?>