<?php
/**
 * Settings class for Workflow settings
 *
 * @copyright   Copyright (c) 2016, Nugget Solutions, Inc
 * @license     http://opensource.org/licenses/gpl-2.0.php GNU Public License
 * @since       2.0
 *
 */

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
   exit();
}

/*
 * OW_Workflow_Settings Class
 *
 * @since 2.0
 */

class OW_Workflow_Settings {

   /**
    * @var string group name
    */
   protected $ow_workflow_group_name = 'ow-settings-workflow';

   /**
    * @var string activate workflow option name
    */
   protected $ow_activate_workflow = 'oasiswf_activate_workflow';

   /**
    * @var string default due days option name
    */
   protected $ow_default_due_days_option_name = 'oasiswf_default_due_days';

   /**
    * @var string show workflow on post types option name
    */
   protected $ow_show_wfsettings_on_post_types_option_name = 'oasiswf_show_wfsettings_on_post_types';

   /**
    * @var string review process setting option name
    */
   protected $ow_review_process_setting_option_name = 'oasiswf_review_process_setting';

   /**
    * Set things up.
    *
    * @since 2.0
    */
   public function __construct() {
      add_action( 'admin_init', array( $this, 'init_settings' ) );
   }

   // White list our options using the Settings API
   public function init_settings() {
      register_setting( $this->ow_workflow_group_name,
      		$this->ow_activate_workflow, array( $this, 'validate_activate_workflow_process' ) );
      register_setting( $this->ow_workflow_group_name,
      		$this->ow_default_due_days_option_name, array( $this, 'validate_default_due_days' ) );
      register_setting( $this->ow_workflow_group_name,
      		$this->ow_show_wfsettings_on_post_types_option_name, array( $this, 'validate_specified_action_by_roles' ) );
      register_setting( $this->ow_workflow_group_name,
      		$this->ow_review_process_setting_option_name, array( $this, 'validate_review_process_setting' ) );
   }

   /**
    * sanitize user data
    * @param string $is_activated
    * @return string
    */
   public function validate_activate_workflow_process( $is_activated ) {
      return sanitize_text_field( $is_activated );
   }

   /**
    *
    * @param string or int $default_due_days
    * @return int
    */
   public function validate_default_due_days( $default_due_days ) {
      // If due days is not empty then do validate and sanitize user input
      $due_days = '';
      if ( ! empty( $default_due_days ) ) {
         if ( is_numeric( $default_due_days ) ) {
            $due_days = intval( sanitize_text_field( $default_due_days ) );
         } else if ( ! is_numeric( $default_due_days ) ) {
            add_settings_error( $this->ow_workflow_group_name,
            		$this->ow_default_due_days_option_name,
            		__( 'Please enter a numeric value for default due date.', 'oasisworkflow' ),
            		'error' );
         } else {
            add_settings_error( $this->ow_workflow_group_name,
            		$this->ow_default_due_days_option_name,
            		__( 'Please enter the number of days for default due date.', 'oasisworkflow' ),
            		'error' );
         }
      }
      return $due_days;
   }

   /**
    * do validate and sanitize selected roles
    * @param array $selected_roles
    * @return array
    */
   public function validate_specified_action_by_roles( $selected_roles ) {
      $roles = array();
      if ( count( $selected_roles ) > 0 ) {

         // Sanitize the value
         $roles = array_map( 'esc_attr', $selected_roles );

         foreach ( $selected_roles as $selected_role ) {
            array_push( $roles, $selected_role );
         }
      }
      return $roles;
   }

   /**
    * sanitize data
    * @param string $review_process_setting
    * @return string
    */
   public function validate_review_process_setting( $review_process_setting ) {
      return sanitize_text_field( $review_process_setting );
   }

   /**
    * generate the page
    *
    * @since 2.0
    */
   public function add_settings_page() {
      $is_activated_workflow = get_option( $this->ow_activate_workflow );
      $default_due_days = get_option( $this->ow_default_due_days_option_name );
      $show_wfsettings_on_post_types = get_option( $this->ow_show_wfsettings_on_post_types_option_name );
      $review_process_setting = get_option( $this->ow_review_process_setting_option_name );
      ?>
      <form id="wf_settings_form" method="post" action="options.php">
          <?php
          settings_fields( $this->ow_workflow_group_name ); // adds nonce for current settings page
          ?>
          <div id="workflow-setting">
              <div id="settingstuff">
                  <div class="select-info">
                      <label class="settings-title"><input type="checkbox"
                      	name="<?php echo $this->ow_activate_workflow; ?>"
                      	value="active" <?php echo ($is_activated_workflow == 'active') ? 'checked' : ''; ?> />
                      	&nbsp;&nbsp;<?php echo __( "Activate Workflow process ?", "oasisworkflow" ); ?>
                      </label>
							 <br />
							 <span class="description">
							 	<?php echo __("(After you are done setting up your editorial workflow, make it available for use by activating the workflow process.)", "oasisworkflow");?>
							 </span>
                  </div>

                  <div class="select-info">
                      <label class="settings-title">
                          <input type="checkbox" id="chk_default_due_days" <?php echo ($default_due_days) ? 'checked' : ''; ?> />
                          &nbsp;&nbsp;
                          <?php echo __( "Set default Due date as CURRENT DATE + ", "oasisworkflow" ); ?>
                      </label>
                      <input type="text" id="default_due_days"
                      	name="<?php echo $this->ow_default_due_days_option_name; ?>"
                      	size="4" class="default_due_days"
                      	value="<?php echo esc_attr( $default_due_days ); ?>"
                      	maxlength=2 />
                      <label class="settings-title"><?php echo __( "day(s).", "oasisworkflow" ); ?></label>
                  </div>

                  <div class="select-info">
                      <div class="list-section-heading">
                          <label>
                          		<?php echo __( "Show Workflow options for the following post/page types:", "oasisworkflow" ) ?>
                          </label>
                      </div>
                      <?php OW_Utility::instance()->owf_checkbox_post_types_multi(
                      		$this->ow_show_wfsettings_on_post_types_option_name . '[]',
                      		$show_wfsettings_on_post_types ); ?>
                  </div>

                  <div class="select-info">
                      <div class="list-section-heading">
                          <label>
                          		<?php echo __( "Set your review setting to dictate how and when item moves to the success step:",
                          			"oasisworkflow" ) ?>
                          	</label>
                      </div>
                      <label for="everyone" class="full-width">
                          <input type="radio" id="everyone"
                          		id="<?php echo $this->ow_review_process_setting_option_name; ?>"
                          		name="<?php echo $this->ow_review_process_setting_option_name; ?>"
                          		value="everyone" <?php echo $review_process_setting == 'everyone' ? 'checked="checked"' : ''; ?>/>
                          		<?php _e( 'Everyone should approve', 'oasisworkflow' ); ?>
                      </label>
                      <label for="more_than_50" class="full-width">
                          <input type="radio" id="more_than_50"
                          		name="<?php echo $this->ow_review_process_setting_option_name; ?>"
                          		value="more_than_50" <?php echo $review_process_setting == 'more_than_50' ? 'checked="checked"' : ''; ?> />
                          		<?php _e( 'More than 50% should approve', 'oasisworkflow' ); ?>
                      </label>
                      <label for="anyone" class="full-width">
                          <input type="radio" id="anyone"
                          		name="<?php echo $this->ow_review_process_setting_option_name; ?>"
                          		value="anyone" <?php echo $review_process_setting == 'anyone' ? 'checked="checked"' : ''; ?> />
                          		<?php _e( 'Anyone should approve', 'oasisworkflow' ); ?>
                      </label>
                  </div>

                  <div class="select-info full-width">
                      <div id="owf_settings_button_bar">
                          <input type="submit" id="settingSave"
                                 class="button button-primary button-large"
                                 value="<?php echo __( "Save", "oasisworkflow" ); ?>" />
                      </div>
                  </div>
              </div>
          </div>
      </form>
      <?php
   }

}

$ow_workflow_settings = new OW_Workflow_Settings();
?>